<?php

namespace Drupal\Tests\media_tableau\FunctionalJavascript\Formatters;

use Drupal\Tests\media_remote\FunctionalJavascript\MediaRemoteFunctionalJavascriptTestBase;

/**
 * Tests the Tableau formatter.
 *
 * @group media_remote
 */
class MediaTableauEmbedFormatterTest extends MediaRemoteFunctionalJavascriptTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'field',
    'field_ui',
    'node',
    'system',
    'media',
    'media_library',
    'media_remote',
    'media_tableau',
  ];

  /**
   * Tests the Tableau formatter.
   */
  public function testTableau() {
    $assert_session = $this->assertSession();
    $session = $this->getSession();
    $page = $session->getPage();

    // Set the formatter on the media source field.
    $source_field_name = $this->mediaType->getSource()->getSourceFieldDefinition($this->mediaType)->getName();
    \Drupal::service('entity_display.repository')
      ->getViewDisplay('media', $this->mediaType->id(), 'default')
      ->setComponent($source_field_name, [
        'type' => 'media_tableau',
      ])
      ->save();

    $node_title = 'Node with a Tableau media';
    $node = $this->drupalCreateNode([
      'type' => $this->nodeType->id(),
      'title' => $node_title,
    ]);
    $this->drupalGet($node->toUrl('edit-form'));
    $this->openMediaLibraryForField('field_node__remote_media');
    // Enter an invalid URL and check the error message.
    $assert_session->elementExists('css', '.media-library-widget-modal input[name="url"]')
      ->setValue('https://drupal.org');
    $assert_session->elementExists('css', '.media-library-widget-modal input[value="Add"]')
      ->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->pageTextContains("The given URL is not valid. Valid values are in the format: https://public.tableau.com/views/[your-tableau-viz-id] or https://public.tableau.com/app/profile/[your-profile-name]/viz/[visualization-name]");
    // Enter a valid URL and save the media item.
    $assert_session->elementExists('css', '.media-library-widget-modal input[name="url"]')
      ->setValue('https://public.tableau.com/views/company-sales_17158979707850/Sheet1');
    $assert_session->elementExists('css', '.media-library-widget-modal input[value="Add"]')
      ->press();
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->pageTextNotContains("The given URL is not valid");
    $media_name = 'Tableau view from https://public.tableau.com/views/company-sales_17158979707850/Sheet1';
    $name_input = $assert_session->waitForElement('css', '.media-library-widget-modal input[name="media[0][fields][name][0][value]"]');
    $this->assertNotNull($name_input);
    $this->assertSame($media_name, $name_input->getValue());

    // Save the modal and the node form.
    $assert_session->pageTextContains('0 of 1 item selected');
    $assert_session->assertWaitOnAjaxRequest();

    $save_btn = $assert_session->waitForElement('xpath', '//button[text()="Save"]');
    $save_btn->click();
    $assert_session->assertWaitOnAjaxRequest();

    $save_btn = $assert_session->waitForElement('xpath', '//button[text()="Save"]');
    $save_btn->click();
    $assert_session->assertWaitOnAjaxRequest();

    $assert_session->pageTextContains('1 of 1 item selected');

    $insert_selected_btn = $assert_session->waitForElement('xpath', '//button[text()="Insert selected"]');
    $insert_selected_btn->click();
    $assert_session->assertWaitOnAjaxRequest();

    $name_in_widget = $assert_session->elementExists('css', '.field--name-field-node__remote-media .media-library-item__name');
    $this->assertSame($media_name, $name_in_widget->getText());
    $page->pressButton('Save');

    $assert_session->pageTextContains("{$this->nodeType->label()} {$node_title} has been updated");
    $embed = $assert_session->elementExists('css', 'article.node--type-node-type .field--name-field-media-media-remote tableau-viz[src^="https://public.tableau.com/"]');
    $this->assertNotNull($embed);
  }

}
